/* *****************************************************************
 * ITS 332 Information Technology II (Networking) Lab
 * Semester 2, 2006
 * SIIT
 *
 * Client/Server Programming Lab
 * File: client_idp.c
 * Date: 29 Jan 2007
 * Version: 1.0
 *
 * Description: 
 * Client to demonstrate UDP sockets programming. You should read the
 * server_udp.c code as well. 
 *
 * Usage:
 * client server_ip_address server_port_number
 *
 * Acknowledgement:
 * This code is based on the examples and descriptions from the 
 * Computer Science Department, Rensselaer Polytechnic Institute at:
 * http://www.cs.rpi.edu/courses/sysprog/sockets/sock.html
 *
 * ***************************************************************** */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

/* ================================================================= */
/* error: display an error message and exit                          */
/* ================================================================= */ 
void error(char *msg)
{
    perror(msg);
    exit(0);
}

/* ================================================================= */
/* main: create socket and send message to server                    */
/* ================================================================= */ 
int main(int argc, char *argv[])
{
   int sock, n;
   struct sockaddr_in server, from;
   struct hostent *hp;
   char buffer[256];
   size_t length;

   /* User must input server and port number */   
   if (argc != 3) { printf("Usage: server port\n");
                    exit(1);
   }
   
   /* Create  a Datagram (UDP) socket */
   sock= socket(AF_INET, SOCK_DGRAM, 0);
   if (sock < 0) error("socket");

   server.sin_family = AF_INET;

   /* Get the IP address for destination server */
   hp = gethostbyname(argv[1]);
   if (hp==0) error("Unknown host");

   /* Set the server address and port */
   bcopy((char *)hp->h_addr, 
        (char *)&server.sin_addr,
         hp->h_length);
   server.sin_port = htons(atoi(argv[2]));

   length=sizeof(struct sockaddr_in);

   /* Prompt for message from user */
   printf("Please enter the message: ");
   bzero(buffer,256);
   fgets(buffer,255,stdin);
   
   /* Send message to socket (server) */
   n=sendto(sock,buffer,
            strlen(buffer),0,(struct sockaddr *) &server,length);
   if (n < 0) error("Sendto");

   /* Receive response from server */
   n = recvfrom(sock,buffer,256,0,(struct sockaddr *) &from, &length);
   if (n < 0) error("recvfrom");

   /* Display response to user */
   write(1,"Got an ack: ",12);
   write(1,buffer,n);
}


