/* *****************************************************************
 * ITS 332 Information Technology II (Networking) Lab
 * Semester 2, 2006
 * SIIT
 *
 * Client/Server Programming Lab
 * File: server_udp.c
 * Date: 29 Jan 2007
 * Version: 1.0
 *
 * Description: 
 * Server to demonstrate UDP sockets programming
 *
 * Usage:
 * server server_port_number
 *
 * Acknowledgement:
 * This code is based on the examples and descriptions from the 
 * Computer Science Department, Rensselaer Polytechnic Institute at:
 * http://www.cs.rpi.edu/courses/sysprog/sockets/sock.html
 *
 * ***************************************************************** */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

/* ================================================================= */
/* error: display an error message and exit                          */
/* ================================================================= */ 
void error(char *msg)
{
    perror(msg);
    exit(0);
}

/* ================================================================= */
/* main: create socket and receive/send to socket                    */
/* ================================================================= */ 
int main(int argc, char *argv[])
{
   int sock, length, n;
   struct sockaddr_in server; /* server address structure */
   struct sockaddr_in from; /* source address structure */
   char buf[1024];
   size_t fromlen;

   /* Port number must be passed as parameter */
   if (argc < 2) {
      fprintf(stderr, "ERROR, no port provided\n");
      exit(0);
   }
   
   /* Create a Datagram (UDP) socket */
   sock=socket(AF_INET, SOCK_DGRAM, 0);
   if (sock < 0) error("Opening socket");

   length = sizeof(server);
   bzero(&server,length);

   /* Set the server address */
   server.sin_family=AF_INET;
   server.sin_addr.s_addr=INADDR_ANY;
   server.sin_port=htons(atoi(argv[1]));

   /* Bind the socket to the address */
   if (bind(sock,(struct sockaddr *)&server,length)<0) 
       error("binding");

   fromlen = sizeof(struct sockaddr_in);
   /* Infinite loop, receiving data and sending response */
   while (1) {
       /* Receive data from socket. Parameters are:
           - server socket
           - buffer to read data into
           - maximum buffer size
           - flags to control the receive operation
           - structure to store source address
           - source address length
           */
       n = recvfrom(sock,buf,1024,0,(struct sockaddr *)&from,&fromlen);
       if (n < 0) error("recvfrom");
       write(1,"Received a datagram: ",21);
       write(1,buf,n);
       /* Write data to socket. Parameters are:
          - server socket
          - data to write
          - length of data
          - flags to control send operation
          - destination address
          - length of destination address
          */
       n = sendto(sock,"Got your message\n",17,
                  0,(struct sockaddr *)&from,fromlen);
       if (n  < 0) error("sendto");
   }
 }

